@@ -9,11 +9,9 @@ from configparser import ConfigParser, NoSectionError
 from distutils.dir_util import copy_tree
 from subprocess import Popen, PIPE
 from typing import Tuple
-
+from github import Github, GithubException
 import git  # type: ignore
-import json
 import os
-import requests
 import shutil
 import tempfile
 from pathlib import Path
@@ -70,12 +68,16 @@ class TemplateSync:
         self.dot_cookietemple = {}
         self.repo_owner = self.gh_username
         self.new_template_version = new_template_version
+        self.github = Github(self.token)
+        self.blacklisted_globs = []
 
     def sync(self):
         """
         Sync the cookietemple project
         """
         self.inspect_sync_dir()
+        # get blacklisted files on current working branch
+        self.blacklisted_globs = self.get_blacklisted_sync_globs()
         self.checkout_template_branch()
         self.delete_template_branch_files()
         self.make_template_project()
@@ -179,7 +181,7 @@ class TemplateSync:
             choose_domain(path=Path.cwd(), domain=None, dot_cookietemple=self.dot_cookietemple)
             # copy into the cleaned TEMPLATE branch's project directory
             log.debug(f'Copying created template into {self.project_dir}.')
-            copy_tree(os.path.join(tmpdirname, self.dot_cookietemple['project_slug']), str(self.project_dir))
+            copy_tree(os.path.join(tmpdirname, self.dot_cookietemple['project_slug_no_hyphen']), str(self.project_dir))
             log.debug(f'Changing directory back to {old_cwd}.')
             os.chdir(old_cwd)
 
@@ -195,26 +197,40 @@ class TemplateSync:
         try:
             # git add only non-blacklisted files
             print('[bold blue]Staging template.')
+            # add all files to stage
             self.repo.git.add(A=True)
+            # get all changed/modified files during the sync (including blacklisted ones)
             changed_files = [item.a_path for item in self.repo.index.diff('HEAD')]
-            globs = self.get_blacklisted_sync_globs()
-            blacklisted_changed_files = []
-            for pattern in globs:
+            blacklisted_changed_files = set()
+            for pattern in self.blacklisted_globs:
                 # keep track of all staged files matching a glob from the cookietemple.cfg file
                 # those files will be excluded from syncing but will still be available in every new created projects
-                blacklisted_changed_files += fnmatch.filter(changed_files, pattern)
+                blacklisted_changed_files |= {file for file in fnmatch.filter(changed_files, pattern)}
             nl = '\n'
             log.debug(f'Blacklisted (unsynced) files are:{nl}{nl.join(file for file in blacklisted_changed_files)}' if blacklisted_changed_files else
                       'No blacklisted files for syncing found.')
-            print('[bold blue]Committing changes of non blacklisted files.')
+            # each blacklisted file must be first unstaged and then get its changes discarded, so it will not be marked as modified by git
+            for blacklisted_file in blacklisted_changed_files:
+                log.debug(f'Unstaging {blacklisted_file}')
+                Popen(['git', 'reset', blacklisted_file], stdout=PIPE, stderr=PIPE, universal_newlines=True)
+                log.debug(f'Discarding changes in working directory of file {blacklisted_file}')
+                Popen(['git', 'checkout', '--', blacklisted_file], stdout=PIPE, stderr=PIPE, universal_newlines=True)
+
             files_to_commit = [file for file in changed_files if file not in blacklisted_changed_files]
             log.debug(f'Files to commit are:{nl}{nl.join(file for file in files_to_commit)}' if files_to_commit else
                       'No files to commit found.')
-            Popen(['git', 'commit', '-m', 'Cookietemple sync', *files_to_commit], stdout=PIPE, stderr=PIPE, universal_newlines=True)
-            print('[bold blue]Stashing and saving TEMPLATE branch changes!')
-            Popen(['git', 'stash'], stdout=PIPE, stderr=PIPE, universal_newlines=True)
-            self.made_changes = True
-            print('[bold blue]Committed changes to TEMPLATE branch')
+            if files_to_commit:
+                print('[bold blue]Committing changes of non blacklisted files.')
+                Popen(['git', 'commit', '-m', 'cookietemple sync', *files_to_commit], stdout=PIPE, stderr=PIPE, universal_newlines=True)
+                print('[bold blue]Stashing and saving TEMPLATE branch changes!')
+                Popen(['git', 'stash'], stdout=PIPE, stderr=PIPE, universal_newlines=True)
+                self.made_changes = True
+                print('[bold blue]Committed changes to TEMPLATE branch')
+
+            # if a file was added to the template, but is blacklisted, it remains untracked; so it should be removed
+            for untracked_file in self.repo.untracked_files:
+                log.debug(f'Removing untracked file {untracked_file}')
+                Path(untracked_file).unlink()
         except Exception as e:
             print(f'[bold red]Could not commit changes to TEMPLATE:\n{e}')
             sys.exit(1)
@@ -222,8 +238,8 @@ class TemplateSync:
 
     def push_template_branch(self):
         """
-        If we made any changes, push the TEMPLATE branch to the default remote
-        and try to make a PR.
+        If there are any changes to the template, push the TEMPLATE branch to the default remote
+        and push to the actual sync temporary branch, where a sync PR is created from to development branch.
         """
         print(f'[bold blue]Pushing TEMPLATE branch to remote: {os.path.basename(self.project_dir)}')
         try:
@@ -233,104 +249,121 @@ class TemplateSync:
             self.repo.head.ref.set_tracking_branch(origin.refs.TEMPLATE)
             log.debug('Pushing to upstream branch TEMPLATE.')
             self.repo.git.push(force=True)
+            print(f'[bold blue]Checking out to new branch cookietemple_sync_v{self.new_template_version}')
+            log.debug(f'git checkout -b cookietemple_sync_v{self.new_template_version}')
+            self.repo.git.checkout('-b', f'cookietemple_sync_v{self.new_template_version}')
+            log.debug(f'git push origin cookietemple_sync_v{self.new_template_version}')
+            print(f'[bold blue]Pushing to remote branch cookietemple_sync_v{self.new_template_version}')
+            self.repo.remotes.origin.push(refspec=f'cookietemple_sync_v{self.new_template_version}:cookietemple_sync_v{self.new_template_version}')
         except git.exc.GitCommandError as e:
-            print(f'Could not push TEMPLATE branch:\n{e}')
+            print(f'Could not push TEMPLATE or cookietemple_sync_v{self.new_template_version} branch:\n{e}')
             sys.exit(1)
 
     def make_pull_request(self):
         """
-        Create a pull request to a base branch from a head branch (default: TEMPLATE)
+        Create a pull request to a base branch from a head branch (that is, a temporary branch only created for syncing the new template version)
         """
         log.debug('Preparing PR contents to submit a sync PR.')
         if self.dot_cookietemple['is_github_orga']:
             self.repo_owner = self.dot_cookietemple['github_orga']
         pr_title = f'Important cookietemple template update {self.new_template_version} released!'
-        pr_body_text = (
-            'A new release of the main template in cookietemple has just been released. '
-            'This automated pull-request attempts to apply the relevant updates to this Project.\n\n'
-            'Please make sure to merge this pull-request as soon as possible. '
-            'Once complete, make a new minor release of your Project.\n\n'
-            'For more information on the actual changes, read the latest cookietemple changelog.')
-        log.debug(f'PR title is {pr_title} and PR body: {pr_body_text}')
+        if self.major_update:
+            pr_body_text = (
+                'A new major release of the corresponding template in cookietemple has just been released. '
+                'This automated pull-request attempts to apply the relevant updates to this Project.\n\n'
+                'This means, that the project template has received significant updates and some new features may be '
+                'difficult to integrate into your current project.\n'
+                'Consider disabling sync by editing the cookietemple.cfg file, if you do not plan on migrating to the new template structure.'
+                'For more information on the actual changes, read the latest cookietemple release notes.')
 
-        # Only create PR if it does not already exist
-        if not self.check_pull_request_exists():
-            self.submit_pull_request(pr_title, pr_body_text)
         else:
-            print('[bold blue]An open cookietemple sync PR already exists at your repo. Changes were added to the existing PR!')
+            pr_body_text = (
+                'A new release of the corresponding template in cookietemple has just been released. '
+                'This automated pull-request attempts to apply the relevant updates to this Project.\n\n'
+                'Please make sure to merge this pull-request as soon as possible. '
+                'Once complete, make a new minor release of your Project.\n\n'
+                'For more information on the actual changes, read the latest cookietemple release notes.')
+        log.debug(f'PR title is:\n{pr_title}')
+        log.debug(f'PR body is:\n{pr_body_text}')
 
-    def submit_pull_request(self, pr_title, pr_body_text):
+        # Check, whether a cookietemple sync PR is already open
+        self.check_pull_request_exists()
+        # Submit the new pull request with the latest cookietemple sync changes
+        self.submit_pull_request(pr_title, pr_body_text)
+
+    def submit_pull_request(self, pr_title: str, pr_body_text: str):
         """
         Create a new pull-request on GitHub
         """
-        pr_content = {
-            'title': pr_title,
-            'body': pr_body_text,
-            'maintainer_can_modify': True,
-            'head': 'TEMPLATE',
-            'base': 'development',
-        }
-        log.debug(f'Trying to submit a sync PR to https://api.github.com/repos/{self.repo_owner}/{self.dot_cookietemple["project_slug"]}/pulls')
-        r = requests.post(
-            url=f'https://api.github.com/repos/{self.repo_owner}/{self.dot_cookietemple["project_slug"]}/pulls',
-            data=json.dumps(pr_content),
-            auth=requests.auth.HTTPBasicAuth(self.gh_username, self.token),
-        )
+        repo = self.github.get_repo(f'{self.repo_owner}/{self.dot_cookietemple["project_slug"]}')
         try:
-            self.gh_pr_returned_data = json.loads(r.content)
-            returned_data_prettyprint = json.dumps(self.gh_pr_returned_data, indent=4)
-        except requests.RequestException:
-            self.gh_pr_returned_data = r.content
-            returned_data_prettyprint = r.content
-
-        # PR worked
-        if r.status_code == 201:
+            repo.create_pull(title=pr_title, body=pr_body_text, head=f'cookietemple_sync_v{self.new_template_version}', base='development',
+                             maintainer_can_modify=True)
             print('[bold blue]Successfully created PR!')
 
         # Something went wrong
-        else:
-            print(f'GitHub API returned code {r.status_code}: \n{returned_data_prettyprint}')
+        except GithubException as e:
+            print(f'[bold red]Could not create a PR due to the following error:\n{e}')
             sys.exit(1)
 
     def check_pull_request_exists(self) -> bool:
         """
-        Check, if a cookietemple sync PR is already pending. If so, just push changes and do not create a new PR!
+        Check, whether a cookietemple sync PR is already open. If so, close this one, as it is outdated.
 
         :return Whether a cookietemple sync PR is already open or not
         """
-        query_url = f'https://api.github.com/repos/{self.repo_owner}/{self.dot_cookietemple["project_slug"]}/pulls?state=open'
-        headers = {'Authorization': f'token {self.token}'}
+        repo = self.github.get_repo(f'{self.repo_owner}/{self.dot_cookietemple["project_slug"]}')
         # query all open PRs
         log.debug('Querying open PRs to check if a sync PR already exists.')
-        r = requests.get(query_url, headers=headers)
-        query_data = r.json()
-        log.debug(f'Query returned: {query_data}')
         # iterate over the open PRs of the repo to check if a cookietemple sync PR is open
-        for pull_request in query_data:
+        for pull_request in repo.get_pulls(state='open'):
             log.debug('Already open sync PR has been found.')
-            if 'Important cookietemple template update' in pull_request['title']:
+            # if an older, outdated cookietemple sync PR is still open, close it first
+            if 'Important cookietemple template update' in pull_request.title:
+                print('[bold blue]Detected open, but outdated cookietemple sync PR. Closing it in favor of the newest sync PR!')
+                pull_request.edit(state='closed')
                 return True
         return False
 
-    def check_sync_level(self) -> bool:
+    def should_run_sync(self) -> bool:
         """
-        Check whether a pull request should be made according to the set level in the cookietemple.cfg file.
+        Check, whether sync should run. This depends on two things:
+        1.) First check, whether the user disabled sync in the cookietemple.cfg file with sync_enabled = False in the [sync] section
+        2.) Secondly, whether the configured level in the [sync_level] section does not restrict the sync.
         Possible levels are:
             - patch: Always create a pull request (lower bound)
             - minor: Create a pull request if it's a minor or major change
             - major: Create a pull request only if it's a major change
-        :return: Whether the changes level is equal to or smaller than the set sync level; whether a PR should be created or not
+        :return: Whether a sync should run determined by the rules above
         """
-        log.debug(f'Checking sync level constraints using parsed results from {self.project_dir}/cookietemple.cfg')
+        log.debug(f'Checking sync rules using parsed results from {self.project_dir}/cookietemple.cfg')
         try:
             parser = ConfigParser()
             parser.read(f'{self.project_dir}/cookietemple.cfg')
+            try:
+                sync_enabled = parser.items('sync')
+            except NoSectionError:
+                print('[bold yellow]Could not find the <sync> section in cookietemple.cfg!')
+                print('[bold yellow]Enabling sync temporarily.')
+                sync_enabled = [('sync_enabled', 'True')]
+            # sync is enabled -> just proceed
+            if sync_enabled[0][1].lower() in {'yes', 'y', 'true'}:
+                pass
+            # sync is disabled -> stop sync
+            elif sync_enabled[0][1].lower() in {'no', 'n', 'false'}:
+                return False
+            # misconfigured sync_enabled with some unknown value
+            else:
+                print(f'[bold blue]Unknown value {sync_enabled[0][1]} for sync_enabled config.\nAllowed values are [bold green]Yes, yes, True, true, Y, y '
+                      f'[bold blue] or [bold green]No, no, False, false, N, n [bold blue]!')
+                return False
+            # now, check for the sync level
             level_item = list(parser.items('sync_level'))
             log.debug(f'Parsing level constraint returned: {level_item}.')
             # check for proper configuration if the sync_level section (only one item named ct_sync_level with valid levels major or minor
             if len(level_item) != 1 or 'ct_sync_level' not in level_item[0][0] or not any(level_item[0][1] == valid_lvl for valid_lvl in
                                                                                           ['major', 'minor', 'patch']):
-                print('[bold red]Your sync_level section is missconfigured. Make sure that it only contains one item named ct_sync_level with only valid levels'
+                print('[bold red]Your sync_level section is misconfigured. Make sure that it only contains one item named ct_sync_level with only valid levels'
                       ' patch, minor or major!')
                 sys.exit(1)
             # check in case of minor update that level is not set to major (major case must not be handled as level is a lower bound)
@@ -343,10 +376,10 @@ class TemplateSync:
             else:
                 log.debug('All updates are allowed because patch level is set.')
                 return True
-        # cookietemple.cfg file was not found or has no section sync_level
+        # cookietemple.cfg file was not found or has no section sync or sync_level
         except NoSectionError:
             print('[bold red]Could not read from cookietemple.cfg file. '
-                  'Make sure your specified path contains a cookietemple.cfg file and has a sync_level section!')
+                  'Make sure your specified path contains a cookietemple.cfg file and has a sync and a sync_level section!')
             sys.exit(1)
 
     def get_blacklisted_sync_globs(self) -> list:
@@ -381,7 +414,7 @@ class TemplateSync:
             sys.exit(1)
 
     @staticmethod
-    def update_sync_token(project_name: str, gh_username='') -> None:
+    def update_sync_token(project_name: str, gh_username: str = '') -> None:
         """
         Update the sync token secret for the repository.
 
@@ -400,7 +433,7 @@ class TemplateSync:
     @staticmethod
     def has_template_version_changed(project_dir: Path) -> Tuple[bool, bool, bool, str, str]:
         """
-        Check, if the cookietemple template has been updated since last check/sync of the user.
+        Check, whether the cookietemple template has been updated since last check/sync of the user.
 
         :return: Both false if no versions changed or a micro change happened (for ex. 1.2.3 to 1.2.4). Return is_major_update True if a major version release
         happened for the cookietemple template (for example 1.2.3 to 2.0.0). Return is_minor_update True if a minor change happened (1.2.3 to 1.3.0).
